@@ -49,6 +49,7 @@
 #include <cap-ng.h>
 #endif
 #include "libaudit.h"
+#include "auplugin.h"
 #include "private.h"
 #include "remote-config.h"
 #include "queue.h"
@@ -119,9 +120,15 @@ gss_ctx_id_t my_context;
 
 /*
  * SIGTERM handler
+ *
+ * Only honor the signal if it comes from the parent process so that other
+ * tasks (cough, systemctl, cough) can't make the plugin exit without
+ * the dispatcher in agreement. Otherwise it will restart the plugin.
  */
-static void term_handler( int sig )
+static void term_handler(int sig, siginfo_t *info, void *ucontext)
 {
+	if (info && info->si_pid != getppid())
+		return;
         stop = 1;
 }
 
@@ -268,6 +275,12 @@ static void safe_exec(const char *exe, const char *message)
 	/* Child */
 	sigfillset (&sa.sa_mask);
 	sigprocmask (SIG_UNBLOCK, &sa.sa_mask, 0);
+#ifdef HAVE_CLOSE_RANGE
+	close_range(3, ~0U, 0); /* close all past stderr */
+#else
+	for (int i=3; i<24; i++)     /* Arbitrary number */
+		close(i);
+#endif
 
 	argv[0] = (char *)exe;
 	argv[1] = (char *)message;
@@ -498,8 +511,6 @@ int main(int argc, char *argv[])
 	sa.sa_flags = 0;
 	sigemptyset(&sa.sa_mask);
 	/* Set handler for the ones we care about */
-	sa.sa_handler = term_handler;
-	sigaction(SIGTERM, &sa, NULL);
 	sa.sa_handler = hup_handler;
 	sigaction(SIGHUP, &sa, NULL);
 	sa.sa_handler = user1_handler;
@@ -508,6 +519,9 @@ int main(int argc, char *argv[])
 	sigaction(SIGUSR2, &sa, NULL);
 	sa.sa_handler = child_handler;
 	sigaction(SIGCHLD, &sa, NULL);
+	sa.sa_sigaction = term_handler;
+	sa.sa_flags = SA_SIGINFO;
+	sigaction(SIGTERM, &sa, NULL);
 	if (load_config(&config, CONFIG_FILE))
 		return 6;
 
@@ -535,7 +549,7 @@ int main(int argc, char *argv[])
 	syslog(LOG_NOTICE, "Audisp-remote started with queue_size: %zu",
 		q_queue_length(queue));
 
-	while (stop == 0) { //FIXME break out when socket is closed
+	while (stop == 0) {
 		fd_set rfd, wfd;
 		struct timeval tv;
 		char event[MAX_AUDIT_MESSAGE_LENGTH];
@@ -595,7 +609,7 @@ int main(int argc, char *argv[])
 		// See if input fd is also set
 		if (FD_ISSET(ifd, &rfd)) {
 			do {
-				if (audit_fgets(event,sizeof(event),ifd) > 0) {
+				if (auplugin_fgets(event,sizeof(event),ifd) > 0) {
 					if (!transport_ok && remote_ended &&
 						(config.remote_ending_action ==
 								FA_RECONNECT ||
@@ -633,9 +647,9 @@ int main(int argc, char *argv[])
 						else
 							queue_error();
 					}
-				} else if (audit_fgets_eof())
+				} else if (auplugin_fgets_eof())
 					stop = 1;
-			} while (audit_fgets_more(sizeof(event)));
+			} while (auplugin_fgets_more(sizeof(event)));
 		}
 		// See if output fd is also set
 		if (sock >= 0 && FD_ISSET(sock, &wfd)) {
@@ -1550,7 +1564,7 @@ static int check_message_ascii(void)
 	char buf[64];
 
 	rc = ar_read(sock, buf, sizeof(buf));
-	if (rc < 0 || remote_ended)
+	if (rc <= 0 || remote_ended)
 		stop = 1;
 
 	return 0;
