@@ -41,6 +41,7 @@
 #include "auparse-idata.h"
 #include "ausearch-nvpair.h"
 
+
 #define NAME_OFFSET 28
 static const char key_sep[2] = { AUDIT_KEY_SEPARATOR, 0 };
 
@@ -209,10 +210,20 @@ int extract_search_items(llist *l)
  */
 static nvlist uid_nvl;
 static int uid_list_created=0;
+static int interp_init = 0;
+static auparse_state_t *au = NULL;
 static const char *lookup_uid(const char *field, uid_t uid)
 {
 	const char *value;
-	value = _auparse_lookup_interpretation(field);
+
+	if (!interp_init) {
+		au = auparse_init(AUSOURCE_BUFFER, "");
+		if (au == NULL)
+			return NULL;
+		interp_init = 1;
+	}
+
+	value = _auparse_lookup_interpretation(au, field);
 	if (value)
 		return value;
 	if (uid == 0)
