@@ -38,6 +38,7 @@
 #include <locale.h>
 #include <signal.h>
 #include "libaudit.h"
+#include "common.h"
 #include "auditd-config.h"
 #include "ausearch-options.h"
 #include "ausearch-lol.h"
@@ -106,6 +107,7 @@ int main(int argc, char *argv[])
 	setrlimit(RLIMIT_FSIZE, &limit);
 	setrlimit(RLIMIT_CPU, &limit);
 	set_aumessage_mode(MSG_STDERR, DBG_NO);
+	set_allow_links(1); // auditd might have -l flag, ausearch should be lenient here
 	(void) umask( umask( 077 ) | 027 );
 
 	/* Load config so we know where logs are and eoe_timeout */
@@ -220,7 +222,6 @@ skip_checkpt:
 	free((char *)event_tuid);
 	free((char *)event_teuid);
 	free((char *)event_tauid);
-	auparse_destroy(NULL);
 	output_auparse_finish();
 	if (rc)
 		return rc;
@@ -244,6 +245,7 @@ static int process_logs(void)
 		clear_config (&config);
 
 		strncpy(dirname, user_file, MAXPATHLEN-32);
+		dirname[MAXPATHLEN-32] = '\0';
 		if (dirname[strlen(dirname)-1] != '/')
 				strcat(dirname, "/");
 		strcat (dirname, "audit.log");
